@@ -8,6 +8,8 @@ Created on Wed Jan  8 15:45:09 2020
 Script splits a fastq into specified chunks
 """
 
+import itertools
+import logging
 from multiprocessing import SimpleQueue
 from typing import Tuple
 import subprocess
@@ -26,18 +28,20 @@ def run_unix_split(fn: os.PathLike,
                    n_cores=1):
 
     statement = []    
-    if fn.endswith(".gz"):
-        cmd = f"""zcat {fn} | split -l {n_reads * 4} -d --additional-suffix=_{read_number}.fastq - {output_prefix}_part;"""
+
+    cmd = f"""zcat {fn} | split FILTER -l {n_reads * 4} -d --additional-suffix=_{read_number}.fastq - {output_prefix}_part;"""
+    
+    if not ".gz" in fn:
+        cmd = cmd.replace("zcat", "cat")
+    
+    if gzip:
+        cmd = cmd.replace("FILTER", f"--filter='pigz -p {n_cores} > $FILE.gz'")
     else:
-        cmd = f"cat {fn} | split -l {n_reads * 4} -d  --additional-suffix=_{read_number}.fastq - {output_prefix}_part;"
+        cmd = cmd.replace("FILTER", "")
     
     statement.append(cmd)
 
-    if gzip:
-        statement.append(
-            f"ls {output_prefix}_part* | xargs -P {n_cores} -n 1 gzip -{compression_level}"
-        )
-
+    logging.info(f"Running: {cmd}")
     subprocess.run(' '.join(statement), shell=True)
 
 
@@ -53,6 +57,8 @@ def split(
     """ 
     Splits fastq file(s) into equal chunks of n reads.
 
+    Will now need "," between files of the same read.
+
     \f
     Args:
      input_files (Tuple): Input fastq files to process.
@@ -107,9 +113,12 @@ def split(
 
     elif method == "unix": # Using unix split to perform the splitting
 
-
         tasks = []
         n_cores_per_task = (n_cores // 2) if (n_cores // 2) > 1 else 1
+
+        if "," in input_files[0]: # Allows for specifying multiple files
+            input_files = [fnames.replace(",", " ") for fnames in input_files]
+
         for ii, fn in enumerate(input_files):
             t = delayed(run_unix_split)(fn, 
                                         n_reads=n_reads,
